import unittest
import smtplib

from mailem import Message, Postman


class TestLoopback(unittest.TestCase):
    def test_smtp(self):
        # Fake
        msg = Message(['test@gmail.com'], 'Test')
        postman = Postman('test@example.com', None)  # It's never used anyway

        # Cannot use `None` as Connection
        self.assertRaises(AttributeError, postman.connect().__enter__)

        # Mock and test
        with postman.loopback() as lo:
            # Send
            with postman.connect() as c:
                c.sendmail(msg)

            # Check
            self.assertEqual(len(lo), 1)

        self.assertEqual(len(lo), 1)  # Still usable

        # Now fails again
        self.assertRaises(AttributeError, postman.connect().__enter__)



# -*- coding: utf-8 -*-

import unittest
import smtplib
import smtpd
import threading
import asyncore
from time import sleep

from mailem import Message, Postman
from mailem.connection import SMTPConnection

try:
    import aiosmtpd

    from aiosmtpd.controller import Controller
    from .asyncio_utils import StashingHandler
except ImportError:
    aiosmtpd = None


class TestSMTP(unittest.TestCase):
    def test_smtp(self):
        # Fake
        msg = Message(['test@gmail.com'], 'Test')
        postman = Postman('test@example.com',
                          SMTPConnection(
                              'smtp.gmail.com', 587,
                              'test@gmail.com', 'wrong',
                              tls=True))

        # try to connect: an error should occur
        try:
            with postman.connect() as c:
                raise AssertionError('Exception not raised')
        except smtplib.SMTPAuthenticationError as e:
            self.assertEqual(e.smtp_code, 535)
            self.assertIn(b'Username and Password not accepted', e.smtp_error)

    smtpd_port = 50587

    @unittest.skip('No need for this test because we have another one.')
    def test_real_mail_aiosmtpd(self):
        """ Test sending messages with a real-world SMTPD server """
        if aiosmtpd is None:
            self.skipTest('aiosmtpd not available')

        # Start an smtp server
        mail_handler = StashingHandler()
        controller = Controller(mail_handler, loop=None,
                                hostname='localhost', port=self.smtpd_port)
        controller.start()

        # Give it time to settle
        sleep(0.5)

        # Initialize a Postman
        postman = Postman('test@example.com',
                          NoLoginSMTP('localhost', self.smtpd_port, None, None))

        # Send messages
        with postman.connect() as c:
            # Send plaintext message
            msg = Message(['test@example.com'], 'Subject', 'HTML message')
            c.sendmail(msg)

            # Send unicode message
            msg = Message(['test@example.com'], u'Håkon', u'Håkon')
            c.sendmail(msg)

        # Done
        controller.stop()

        # Test
        self.assertEqual(len(mail_handler.mail), 2)

    # TODO: remove this test when Python 2 becomes obsolete
    def test_real_mail_smtpd(self):
        """ Test sending messages with a real SMTPD server """
        # port+1 because python can't let it go.. :) hack?
        smtp_server = TestingSMTPServer(port=self.smtpd_port+1)
        smtp_server.start()
        sleep(0.5)

        # Initialize a Postman
        postman = Postman('test@example.com',
                          NoLoginSMTP('localhost', self.smtpd_port+1, None, None))

        # Send messages
        with postman.connect() as c:
            # Send unicode message
            c.sendmail(Message(['test@example.com'], u'Håkon', u'Håkon'))

        # Test
        self.assertIsNotNone(smtp_server.received_data)

        # Clean-up
        smtp_server.close()
        smtp_server.join()


class NoLoginSMTP(SMTPConnection):
    # aiosmtpd does not support AUTH: we can't login()
    # Thus, override it with a method that connects without authentication
    def connect(self):
        self.client = self._get_client()


class TestingSMTPServer(smtpd.SMTPServer, threading.Thread):
    """ smtpd lib server """

    def __init__(self, port=25):
        self.received_peer = None
        self.received_mailfrom = None
        self.received_rcpttos = None
        self.received_data = None

        smtpd.SMTPServer.__init__(
            self,
            ('localhost', port),
            ('localhost', port)
        )
        threading.Thread.__init__(self)

    def process_message(self, peer, mailfrom, rcpttos, data, **kwargs):
        self.received_peer = peer
        self.received_mailfrom = mailfrom
        self.received_rcpttos = rcpttos
        self.received_data = data

        self.close()  # causes asyncore loop to quit

    def run(self):
        try:
            asyncore.loop()
        except asyncore.ExitNow:
            pass



# -*- coding: utf-8 -*-

import unittest

from mailem import Attachment, ImageAttachment
from mailem.template import Template, TemplateRegistry
from mailem.template.renderer import Jinja2TemplateRenderer

from future.utils import PY2


class TemplateTest(unittest.TestCase):
    def _check_signup_template(self, signup):
        msg = signup(['kolypto@gmail.com'], dict(user='Honored User'), attachments=[
            Attachment('kolypto.gpg', '123'),
        ])

        # Accessing protected fields is discouraged, but still, let's make sure they're alright
        self.assertEqual(msg._subject, u'Hello Honored User °C')
        self.assertEqual(msg._html,
                         u'You are signed up -- <img src="cid:flower.jpg" /> localhost\n')
        self.assertEqual(msg._text, None)
        self.assertEqual(msg._sender, None)
        self.assertEqual(msg._cc, [])
        self.assertEqual(msg._bcc, [])
        self.assertEqual(msg._reply_to, None)

        # MIME message
        msg_str = str(msg)

        self.assertIn('Subject: =?utf-8?q?Hello_Honored_User_=C2=B0C?=', msg_str)
        self.assertIn('To: kolypto@gmail.com', msg_str)
        self.assertIn('You are signed up -- <img src="cid:flower.jpg" /> localhost', msg_str)

        if PY2:
            self.assertIn('Content-Disposition: inline; filename="=?utf-8?q?flower=2Ejpg?="', msg_str)
        else:
            self.assertIn('Content-Disposition: inline; filename="flower.jpg"', msg_str)
        #self.assertIn('Content-Type: image/jpeg', msg_str)
        self.assertIn('Content-ID: <flower.jpg>', msg_str)

        if PY2:
            self.assertIn('Content-Disposition: attachment; filename="=?utf-8?q?kolypto=2Egpg?="', msg_str)
        else:
            self.assertIn('Content-Disposition: attachment; filename="kolypto.gpg"', msg_str)
        self.assertIn('Content-Type: application/octet-stream', msg_str)

    def test_template(self):
        """ Test Template """
        signup = Template(
            u'Hello $user °C',
            'You are signed up -- <img src="cid:flower.jpg" /> $domain\n',
            attachments=[
                ImageAttachment('flower.jpg', b'\xff\xd8\xff\xe0\x00\x10JFIF', 'inline'),
            ],
            defaults=dict(domain='localhost')
        )
        self._check_signup_template(signup)

    def test_jinja2(self):
        """ Test jinja2 template renderer """
        signup = Template(
            u'Hello {{ user }} °C',
            'You are signed up -- <img src="cid:flower.jpg" /> {{ domain }}\n',
            attachments=[
                ImageAttachment('flower.jpg', b'\xff\xd8\xff\xe0\x00\x10JFIF', 'inline'),
            ],
            defaults=dict(domain='localhost')
        )
        signup.set_renderer(Jinja2TemplateRenderer)
        self._check_signup_template(signup)

    def test_from_directory(self):
        """ Test Template.from_directory() """
        signup = Template.from_directory('tests/data/signup')
        signup.defaults(dict(domain='localhost'))
        self._check_signup_template(signup)

    def test_registry(self):
        """ Test TemplateRegistry """
        registry = TemplateRegistry.from_directory('tests/data').defaults(dict(domain='localhost'))
        self._check_signup_template(registry.get('signup'))



# -*- coding: utf-8 -*-

import unittest

from mailem import Message, Attachment, ImageAttachment

from future.utils import PY2


class MessageTest(unittest.TestCase):
    def test_html(self):
        msg = Message(
            ['kolypto@gmail.com'],
            u"Mail'em test °C",
            u"<b>yeah baby, it works!</b>",
            attachments=[
                Attachment(u'test.txt', 'abc')
            ],
            sender=('a@b', 'A'),
            cc=[('a@b', 'A'), 'b@b'],
            bcc=[('a@b', 'A'), 'b@b'],
            reply_to='z@b',
            headers={'Custom': 'Test'}
        )
        msg_str = str(msg)

        self.assertIn('multipart/mixed', msg_str)
        self.assertNotIn('multipart/alternative', msg_str)
        self.assertIn('text/html', msg_str)
        self.assertNotIn('text/plain', msg_str)

        self.assertIn('application/octet-stream', msg_str)
        self.assertIn('Content-Disposition: attachment; filename="test.txt"', msg_str)

        self.assertIn('Subject: =?utf-8?q?Mail=27em_test_=C2=B0C?=', msg_str)
        self.assertIn('To: kolypto@gmail.com', msg_str)
        if PY2:
            self.assertIn('From: =?utf-8?q?A?= <a@b>', msg_str)
            self.assertIn('Cc: =?utf-8?q?A?= <a@b>, b@b', msg_str)
            self.assertIn('Bcc: =?utf-8?q?A?= <a@b>, b@b', msg_str)
        else:
            self.assertIn('From: A <a@b>', msg_str)
            self.assertIn('Cc: A <a@b>, b@b', msg_str)
            self.assertIn('Bcc: A <a@b>, b@b', msg_str)
        self.assertIn('Reply-To: z@b', msg_str)
        self.assertIn('Message-ID:', msg_str)
        self.assertIn('Custom: Test', msg_str)
        self.assertIn('', msg_str)

    def test_html_text(self):
        msg = Message(
            ['kolypto@gmail.com'],
            u"Mail'em test",
            u'Mailtest',
            u"<b>yeah baby, it works!</b>",
            attachments=[
                Attachment(u'test.txt', 'abc')
            ]
        )
        msg_str = str(msg)

        self.assertIn('multipart/mixed', msg_str)
        self.assertIn('multipart/alternative', msg_str)
        self.assertIn('text/html', msg_str)
        self.assertIn('text/plain', msg_str)

        self.assertIn('Content-Type: application/octet-stream', msg_str)
        self.assertIn('Content-Disposition: attachment; filename="test.txt"', msg_str)

    def test_html_image(self):
        msg = Message(
            ['kolypto@gmail.com'],
            u"Mail'em test with inline images",
            u"Cute: <img src='cid:cute.jpg' />",  # cid:<filename>
            attachments=[
                ImageAttachment('cute.jpg', b'\xff\xd8\xff\xe0\x00\x10JFIF', 'inline')
            ]
        )
        msg_str = str(msg)

        self.assertIn('multipart/mixed', msg_str)
        self.assertNotIn('multipart/alternative', msg_str)
        self.assertIn('text/html', msg_str)
        self.assertNotIn('text/plain', msg_str)

        self.assertIn('Content-Type: image/jpeg', msg_str)
        self.assertIn('Content-ID: <cute.jpg>', msg_str)
        if PY2:
            self.assertIn('Content-Disposition: inline; filename="=?utf-8?q?cute=2Ejpg?="', msg_str)
        else:
            self.assertIn('Content-Disposition: inline; filename="cute.jpg"', msg_str)



import os

from .template import Template


class TemplateRegistry(object):
    """ E-Mail template registry.

    Simply contains all your templates and allows to render these by name.
    Useful if you have multiple templates in your app and want to have them prepared.

    Initially, the registry is empty, and you add [`Template`](#template) objects one by one:

    ```python
    from mailem.template import Template, TemplateRegistry

    templates = TemplateRegistry()
    templates.add('signup', Template(
                'Congrats $user, you\'ve signed up!',
               'Welcome to our website!<br> -- $domain',
    ))
    templates.defaults(dict(domain='example.com'))  # set defaults on all templates
    ```

    Alternatively, you can use [`TemplateRegistry.from_directory()`](#templateregistryfrom_directory) to load templates
    from filesystem.

    Now, to render a template, you [`get()`](#templateregistryget) it by name:

    ```python
    msg = templates.get('signup')(['user@gmail.com'], dict(user='Honored User',))
    ```
    """

    @classmethod
    def from_directory(cls, path, **kwargs):
        """ Convenience method to construct a template registry
        with a directory where each template is in a subdirectory

        :param path: Path to templates
        :type path: str
        :param kwargs: Arguments to [Template.from_directory()](#templatefrom_directory), if required
        :rtype: mailem.template.registry.TemplateRegistry
        """
        registry = cls()
        for template_name in os.listdir(path):
            if template_name.startswith('.'):
                continue
            template_path = os.path.join(path, template_name)
            registry.add(template_name, Template.from_directory(template_path, **kwargs))
        return registry

    def __init__(self):
        self._templates = {}
        self._renderer = None
        self._defaults = {}

    def defaults(self, values):
        """ Set default values on all templates.

        New values will overwrite the previous.

        Can be called both before adding templates and after.

        :param values: Default template values
        :type values: dict
        """
        self._defaults.update(values)

        # Set it on all existing templates
        for t in self._templates.values():
            t.defaults(self._defaults)

        # Finish
        return self

    def set_renderer(self, renderer, **kwargs):
        """ Set renderer to be used with all templates.

        Can be called both before adding templates and after.

        :param renderer: Renderer class to use
        :type renderer: type
        :param kwargs: Additional arguments for the renderer
        :type kwargs: dict
        """
        assert self._renderer is None, 'Cannot re-assign a renderer'

        # Store
        self._renderer = renderer

        # Set it on all existing templates
        for t in self._templates.values():
            if not t._renderer:
                t.set_renderer(self._renderer, **kwargs)

        # Finish
        return self

    def add(self, name, template):
        """ Register a template

        :param template: Template object
        :type template: mailem.template.Template
        :returns: The added template (in case you want to set something on it)
        :rtype: mailem.template.Template
        """
        assert name not in self._templates, 'Duplicate template name'

        # Store
        self._templates[name] = template

        # Apply settings
        if self._renderer:
            template.set_renderer(self._renderer)
        template.defaults(self._defaults)

        # Finish
        return template

    def get(self, name):
        """ Get a Template by name

        :param name: Template name
        :type name: str
        :rtype: mailem.template.Template
        :raises KeyError: unknown template name
        """
        return self._templates[name]



# Used in smtp-test: mail handler
# Moved here because `async def` is a syntax error in < 3.7

class StashingHandler(object):
    def __init__(self):
        self.mail = []

    async def handle_RCPT(self, server, session, envelope, address, rcpt_options):
        if not address.endswith('@example.com'):
            return '550 not relaying to that domain'
        envelope.rcpt_tos.append(address)
        return '250 OK'

    async def handle_DATA(self, server, session, envelope):
        self.mail.append(
            (envelope.mail_from, envelope.content.decode('utf8', errors='replace'))
        )
        return '250 Message accepted for delivery'
